% Create pendulum1.m and pendulum2.m files before executing this program

clc
clear all
close all
disp('Refer Example 7.2');
% The axis commands in the code have been included as comments.
% First run the program as it is. Then uncomment the axis commands
% to make them regular MATLAB commands. This will give a zooming effect on
% the plots. Rerun the program to see the zoomed plots.
global A b c k L
global m l g a u Fc

% Basic data
M = 1;  m = 0.15;   % Mass of Cart and Mass of the pendulum
l = 0.5;            % Length of pendulum
g = 9.81;           % Gravitational acceleration
a = 1/(M+m);
b = 4/3-m*a;
Fc = 0.05;

% Creation of  state space matrices for linear model (from analytical development)     
A = [0 1 0 0; g/(l*b) 0 0 0; 0 0 0 1; -m*a*g/b 0 0 0];
b = [0;-a/(l*b);0;(4*a)/(3*b)];
c = [1 0 0 0;0 0 1 0];
d = [0;0];

sys=ss(A,b,c,d);
polesOL=pole(sys)
disp('As one of the poles is in the right-half of s-plane, the system is unstable');
U=ctrb(A,b);
ControllablePoles=rank(U)

% Regulator design
polesCL=[ -0.7999+j*1.5618 -0.7999-j*1.5618 -10 -11];
disp('Regulator gains corresponding to the given poles  :');
k=acker(A,b,polesCL)
RegulatorPoles=eig(A-b*k)

% Simulation
% Linear
x0=[0.1 0 0 0]';
t=[0:0.01:30];
sysCL=ss(A-b*k,[0 0 0 0]',c,d);
[y,t,x1]=lsim(sysCL,0*t,t,x0);
figure(1)
subplot(2,2,1);plot(t,x1(:,1));grid;legend('x_1');
title('                             Response of linear model with controller');
subplot(2,2,2);plot(t,x1(:,2));grid;legend('x_2');
subplot(2,2,3);plot(t,x1(:,3));grid;legend('x_3');
subplot(2,2,4);plot(t,x1(:,4));grid;legend('x_4');
% Nonliner
[t,x2] = ode23('pendulum1',t,x0);
figure(2)
subplot(2,2,1);plot(t,x2(:,1));grid;legend('x_1');
title('                             Response of nonlinear model with controller');
% axis([20 30 -0.0000003 0.0000003]);
subplot(2,2,2);plot(t,x2(:,2));grid;legend('x_2');
% axis([20 30 -0.0000015 0.0000015]);
subplot(2,2,3);plot(t,x2(:,3));grid;legend('x_3');
% axis([20 30 0.0020938-0.0000002 0.0020938+0.0000002]);
subplot(2,2,4);plot(t,x2(:,4));grid;legend('x_4');
% axis([20 30 -0.0000015 0.0000015]);
% Observer Design
ObserverPoles = [-2 -2+j -2-j -3];
disp('Observer gains corresponding to the  given poles :');
% The text uses letter m for Observer gains;we have used L releasing m for pendulum mass
L = place(A',c',ObserverPoles);
L = L'
% Simulation
% Linear
x0 = [0.1 0 0 0 0 0 0 0]';
A1=[A  -b*k ;L*c A-b*k-L*c];
sysCL=ss(A1,[0 0 0 0 0 0 0 0]',[c zeros(size(c))],d);
[y,t,x3]=lsim(sysCL,0*t,t,x0);
figure(3)
subplot(2,2,1);plot(t,x3(:,1));grid;legend('x_1');
% axis([20 30 -0.0000005 0.0000005]);
title('                             Response of linear model with controller and observer');
subplot(2,2,2);plot(t,x3(:,2));grid;legend('x_2');
% axis([20 30 -0.000005 0.000005]);
subplot(2,2,3);plot(t,x3(:,3));grid;legend('x_3');
% axis([20 30 -0.0000001 0.0000001]);
subplot(2,2,4);plot(t,x3(:,4));grid;legend('x_4');
% axis([20 30 -0.0000015 0.0000015]);
% Nonlinear
[t,x4] = ode23('pendulum2',t,x0);
figure(4)
subplot(2,2,1);plot(t,x4(:,1));grid;legend('x_1');
% axis([20 30 -0.0000005 0.0000005]);
title('                             Response of nonlinear model with controller and observer');
subplot(2,2,2);plot(t,x4(:,2));grid;legend('x_2');
% axis([20 30 -0.000005 0.000005]);
subplot(2,2,3);plot(t,x4(:,3));grid;legend('x_3');
% axis([20 30 0.1110057-0.0000003 0.1110057+0.0000003]);
subplot(2,2,4);plot(t,x4(:,4));grid;legend('x_4');
% axis([20 30 -0.0000015 0.0000015]);