% two link robot simulation
% this function is called from the main program to solve the ODE
% it contains the dynamics of the robotic arm

function xdot=robosolver(t,x)
global m1 m2 L1 L2 g
alpha=(m1+m2)*L1^2;
beta=m2*L2*L2;
eta=m2*L1*L2;
e1=g/L1;

% computation of M and N matrices
M11=alpha+beta+2*eta*cos(x(2));
M12=beta+eta*cos(x(2));
M21=M12;
M22=beta;
M=[M11 M12;M21 M22];

N1=-eta*(2*x(3)*x(4)+x(4)*x(4))*sin(x(2)) + alpha*e1*cos(x(1))+eta*e1*cos(x(1)+x(2));
N2=eta*x(3)*x(3)*sin(x(2))+eta*e1*cos(x(1)+x(2));
N=[N1;N2];

% inversion of M matrix
det= M11*M22-M12*M12;
Mi11=M22/det;
Mi12=-M12/det;
Mi22=M11/det;

%reference signals
ref1 = sin(pi*t);
ref2 = cos(pi*t);
ref1Dot = pi*cos(pi*t);
ref2Dot = -pi*sin(pi*t);

X1d = [ref1;ref2];
X2d = [ref1Dot;ref2Dot];
X2dDot =[-pi^2*sin(pi*t);-pi^2*cos(pi*t)];

%error calculation
E1 = X1d - [x(1);x(2)];
E2 = X2d - [x(3);x(4)];
Err1 = E1(1);
Err2 = E1(2);

%calculating torque input by optimal control
tow = Controller(E1,E2,X2dDot,M,N);
tow1 = tow(1);
tow2 = tow(2);

% state equations 
xdot      =[x(3);
            x(4);
            Mi11*(-N1+tow1)+Mi12*(-N2+tow2);
            Mi12*(-N1+tow1)+Mi22*(-N2+tow2);];
        