%Program for self tuning regulator through GPC
%First order system y(k+1)=f*y(k)+g*u(k)

clear all
close all
clc;

%f0 & g0 are true parameters 
f0=1.1052;
g0=0.0526;

%initial output of the system
y(1)=0;

%number of sampling instants upto which system is to be run
final_time=250;

%initial p for RLS estimation
p=1e+8*eye(2,2);

%GPC parameters
global N2;
global Nu;
Nu=1;
N2=4;
rho=.1;

%reference input
for i=1:500
    r(i)=10*square(2*pi*i/200);
end

%initial estimates of f0 and g0
theta_hat=[0.90;0.04];

%input applied to the system at the begining
u(1)=r(1);

for i=1:final_time     
    
    %output of the system at the next sampling instant
    y(i+1)=(f0)*y(i)+(g0)*u(i);
    
    %determination of numerator and denominator of TF
    A=[1 -theta_hat(1,1)];
    B=[theta_hat(2,1)];
    
    Ytest=y(i+1);
    
    %function to generate gamma and phi, to be used in GPC
    [gamma phi]=gen_phiG(A,B,Ytest,y(1:i),u);
    
    %future reference input
    for hor=1:N2
        R(hor)=r(i+hor);
    end
    
    %evaluation of future control input increments
    U=inv(gamma'*gamma+rho*eye(Nu))*gamma'*(R-phi)';
    
    %input at next time step;
    u(i+1)=u(i)+U(1);
    
    %updation of estimates by RLS
    phi1=[y(i) u(i)];
    K=p*(phi1)'*inv(1+phi1*p*(phi1)');
    theta_hat=theta_hat+K*(Ytest-phi1*theta_hat);
    p=(1-K*phi1)*p;
end

figure(1)
plot(u)
%title('Control input to the plant');
ylabel('Control signal')
xlabel('Sampling instants')
figure(2)
plot(y,'r');
title('Tracking performance')
%ylabel('Amplitude')
axis([0 250 -15 30])
xlabel('sampling instants')
hold on
plot(r,'--')
legend('Plant output','Command signal')